@@ -1,17 +1,19 @@
 import sys
 from collections import defaultdict
 from enum import Enum
-from functools import partial
+from functools import partial, wraps
 from types import new_class
 from typing import (
+    TYPE_CHECKING,
+    Any,
     Callable,
     List,
     MutableMapping,
     Optional,
-    TYPE_CHECKING,
     Type,
     TypeVar,
     Union,
+    cast,
     overload,
 )
 
@@ -19,8 +21,8 @@ from apischema.cache import CacheAwareDict
 from apischema.conversions import LazyConversion
 from apischema.conversions.conversions import (
     AnyConversion,
-    ConvOrFunc,
     Conversion,
+    ConvOrFunc,
     resolve_conversion,
 )
 from apischema.conversions.utils import Converter, is_convertible
@@ -29,6 +31,7 @@ from apischema.type_names import type_name
 from apischema.types import AnyType
 from apischema.typing import is_type_var
 from apischema.utils import get_args2, get_origin_or_type, stop_signature_abuse
+from apischema.validation.errors import ValidationError
 
 if TYPE_CHECKING:
     pass
@@ -176,11 +179,30 @@ def reset_serializer(cls: Type):
     _serializers.pop(cls, ...)
 
 
+Func = TypeVar("Func", bound=Callable)
+
+
+class ValueErrorCatcher:
+    def __init__(self, func: Callable[[Any], Any]):
+        wraps(func)(self)
+        self.func = func
+
+    def __call__(self, arg):
+        try:
+            return self.func(arg)
+        except ValueError as err:
+            raise ValidationError(str(err))
+
+
+def catch_value_error(func: Func) -> Func:
+    return cast(Func, ValueErrorCatcher(func))
+
+
 Cls = TypeVar("Cls", bound=type)
 
 
 def as_str(cls: Cls) -> Cls:
-    deserializer(Conversion(cls, source=str))
+    deserializer(Conversion(catch_value_error(cls), source=str, target=cls))
     serializer(Conversion(str, source=cls))
     return cls
 
