@@ -33,6 +33,8 @@ from apischema.objects import ObjectField
 from apischema.ordering import Ordering
 from apischema.schemas import Schema
 from apischema.serialization import (
+    IDENTITY_METHOD,
+    METHODS,
     PassThroughOptions,
     SerializationMethod,
     SerializationMethodVisitor,
@@ -41,8 +43,8 @@ from apischema.serialization.serialized_methods import (
     ErrorHandler,
     SerializedMethod,
     _get_methods,
-    serialized as register_serialized,
 )
+from apischema.serialization.serialized_methods import serialized as register_serialized
 from apischema.types import AnyType, NoneType, Undefined
 from apischema.typing import is_type
 from apischema.utils import (
@@ -62,20 +64,37 @@ from apischema.validation.errors import ValidationError
 class PartialSerializationMethodVisitor(SerializationMethodVisitor):
     use_cache = False
 
+    def __init__(
+        self,
+        aliaser: Aliaser,
+        default_conversion: DefaultConversion,
+        pass_through_options: PassThroughOptions,
+    ):
+        super().__init__(
+            False,
+            aliaser,
+            False,
+            default_conversion,
+            False,
+            False,
+            False,
+            False,
+            True,
+            pass_through_options,
+        )
+
     @property
     def _factory(self) -> Callable[[type], SerializationMethod]:
-        return lambda _: identity
+        raise NotImplementedError
 
     def enum(self, cls: Type[Enum]) -> SerializationMethod:
-        return identity
+        return IDENTITY_METHOD
 
     def object(self, tp: AnyType, fields: Sequence[ObjectField]) -> SerializationMethod:
-        return identity
+        return IDENTITY_METHOD
 
     def visit(self, tp: AnyType) -> SerializationMethod:
-        if tp is UndefinedType:
-            return lambda obj: None
-        return super().visit(tp)
+        return METHODS[NoneType] if tp is UndefinedType else super().visit(tp)
 
 
 @cache
@@ -87,15 +106,7 @@ def partial_serialization_method_factory(
     @lru_cache()
     def factory(tp: AnyType) -> SerializationMethod:
         return PartialSerializationMethodVisitor(
-            additional_properties=False,
-            aliaser=aliaser,
-            check_type=False,
-            default_conversion=default_conversion,
-            exclude_defaults=False,
-            exclude_none=False,
-            exclude_unset=False,
-            fall_back_on_any=False,
-            pass_through_options=PassThroughOptions(),
+            aliaser, default_conversion, PassThroughOptions()
         ).visit_with_conv(tp, conversion)
 
     return factory
@@ -291,16 +302,16 @@ def resolver_resolve(
     if not serialized:
         serialize_result = identity
     elif is_async(resolver.func):
-        serialize_result = as_async(method_factory(types["return"]))
+        serialize_result = as_async(method_factory(types["return"]).serialize)
     else:
-        serialize_result = method_factory(types["return"])
+        serialize_result = method_factory(types["return"]).serialize
     serialize_error: Optional[Callable[[Any], Any]]
     if error_handler is None:
         serialize_error = None
     elif is_async(error_handler):
-        serialize_error = as_async(method_factory(resolver.error_type()))
+        serialize_error = as_async(method_factory(resolver.error_type()).serialize)
     else:
-        serialize_error = method_factory(resolver.error_type())
+        serialize_error = method_factory(resolver.error_type()).serialize
 
     def resolve(__self, __info, **kwargs):
         values = {}
