@@ -20,7 +20,7 @@ from apischema.metadata import conversion
 from apischema.metadata.keys import (
     DEFAULT_AS_SET_METADATA,
     FALL_BACK_ON_DEFAULT_METADATA,
-    FLATTENED_METADATA,
+    FLATTEN_METADATA,
     INIT_VAR_METADATA,
     POST_INIT_METADATA,
     PROPERTIES_METADATA,
@@ -54,7 +54,7 @@ INVALID_METADATA = {
     DEFAULT_AS_SET_METADATA,
     FALL_BACK_ON_DEFAULT_METADATA,
     INIT_VAR_METADATA,
-    FLATTENED_METADATA,
+    FLATTEN_METADATA,
     POST_INIT_METADATA,
     PROPERTIES_METADATA,
     REQUIRED_METADATA,
